/***********************************************************************************************/
/******** DANGER !! DANGER !! DANGER !! DO NOT EDIT FILE -- IT WAS GENERATED AUTOMATICALLY !!! */
/***********************************************************************************************/

/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

/*-----------------------------------------------------------------------------
 Includes
 -----------------------------------------------------------------------------*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "sph_zpl_intensity_flat_imaging.h"
#include "sph_common_keywords.h"
#include "sph_zpl_keywords.h"
#include "sph_zpl_tags.h"
#include "sph_master_frame.h"
#include "sph_error.h"
#include "sph_cube.h"
#include "sph_utils.h"

#include <math.h>
#include <assert.h>
#include <cpl.h>

/*-----------------------------------------------------------------------------
  Error Codes
 -----------------------------------------------------------------------------*/

sph_error_code SPH_ZPL_INTENSITY_FLAT_IMAGING_GENERAL              = SPH_ZPL_INTENSITY_FLAT_IMAGING_ERR_START + 0;
sph_error_code SPH_ZPL_INTENSITY_FLAT_IMAGING_NO_VALID_DIT_LIST    = SPH_ZPL_INTENSITY_FLAT_IMAGING_ERR_START + 1;

sph_error_code SPH_ZPL_INTENSITY_FLAT_IMAGING_PARAMETER_MISSING    = SPH_ZPL_INTENSITY_FLAT_IMAGING_ERR_START + 3;
sph_error_code SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING       = SPH_ZPL_INTENSITY_FLAT_IMAGING_ERR_START + 4;

/*-----------------------------------------------------------------------------
 Private function prototypes
 -----------------------------------------------------------------------------*/

static int sph_zpl_intensity_flat_imaging_create_plugin ( cpl_plugin * );
static int sph_zpl_intensity_flat_imaging_exec_plugin ( cpl_plugin * );
static int sph_zpl_intensity_flat_imaging_destroy_plugin ( cpl_plugin * );

/*-----------------------------------------------------------------------------
 Static variables
 -----------------------------------------------------------------------------*/

static const char* sph_zpl_intensity_flat_imaging_help = 
    "RECIPE INPUT FRAMES: \n"
    "-------------------- \n"
    "Tag                           Min,Max   Description\n"
    "ZPL_INT_FLAT_FIELD_IMAGING_RAW  0, -1   I. NEW STYLE (split format) -- The raw frames in the imaging mode with a primary HDU and image:\n"
    "                                        - represents data cube of NDITS frames from camera-1/2 for a given DIT, including prescan/overscan area of 2 ADUs.\n"
    "                                        II. OLD STYLE -- The raw frames in the image mode with the two extensions  fits file format:\n"
    "                                        - first extension represents data cube of NDITS frames from camera-1 for a given DIT, including prescan/overscan area of 2 ADUs;\n"
    "                                        - second extension represents data cube of NDITS zimpol frames from camera-2 for a given DIT, including prescan/overscan area of 2 ADUs\n"
    "ZPL_INT_FLAT_FIELD_IMAGING_PREPROC_RAW  0, -1   The pre-processed raw intensity flat field frames, imaging mode. To be taken with different illumination levels.\n"
    "                                        Format: ZPL EXP IMAGING = [Im(2),Bt(1)]\n"
    "ZPL_INT_FLAT_FIELD_IMAGING_PREPROC_CAM1  0, -1   The pre-processed raw intensity flat field frames, imaging mode. To be taken with different illumination levels.\n"
    "                                        Format: ZPL EXP IMAGING = [Im(2),Bt(1)]\n"
    "ZPL_INT_FLAT_FIELD_IMAGING_PREPROC_CAM2  0, -1   The pre-processed raw intensity flat field frames, imaging mode. To be taken with different illumination levels.\n"
    "                                        Format: ZPL EXP IMAGING = [Im(2),Bt(1)]\n"
    "ZPL_MASTER_BIAS_IMAGING         0,  1   The master bias imaging frame. Should have readout mode as the pre-processed intensity flat field frames.\n"
    "                                        Format: DOUBLE IMAGE = [Im(8)]\n"
    "ZPL_MASTER_BIAS_IMAGING_CAM1    0,  1   The master bias imaging frame. Should have readout mode as the pre-processed intensity flat field frames.\n"
    "                                        Format: DOUBLE IMAGE = [Im(8)]\n"
    "ZPL_MASTER_BIAS_IMAGING_CAM2    0,  1   The master bias imaging frame. Should have readout mode as the pre-processed intensity flat field frames.\n"
    "                                        Format: DOUBLE IMAGE = [Im(8)]\n"
    "ZPL_MASTER_DARK_IMAGING         0,  1   The master dark imaging frame. Should have readout mode as the pre-processed intensity flat field frames.\n"
    "                                        Format: DOUBLE IMAGE = [Im(8)]\n"
    "ZPL_MASTER_DARK_IMAGING_CAM1    0,  1   The master dark imaging frame. Should have readout mode as the pre-processed intensity flat field frames.\n"
    "                                        Format: DOUBLE IMAGE = [Im(8)]\n"
    "ZPL_MASTER_DARK_IMAGING_CAM2    0,  1   The master dark imaging frame. Should have readout mode as the pre-processed intensity flat field frames.\n"
    "                                        Format: DOUBLE IMAGE = [Im(8)]\n"
    "ZPL_STATIC_BADPIXELMAP_IMAGING  0,  1   Optional frame of static bad pixels. If provided, this is used in addtion to\n"
    "                                        the badpixels from the master dark.\n"
    "                                        Format: ZPL EXP IMAGING = [Im(2),Bt(1)]\n"
    "ZPL_STATIC_BADPIXELMAP_IMAGING_CAM1  0,  1   Optional frame of static bad pixels. If provided, this is used in addtion to\n"
    "                                        the badpixels from the master dark.\n"
    "                                        Format: ZPL EXP IMAGING = [Im(2),Bt(1)]\n"
    "ZPL_STATIC_BADPIXELMAP_IMAGING_CAM2  0,  1   Optional frame of static bad pixels. If provided, this is used in addtion to\n"
    "                                        the badpixels from the master dark.\n"
    "                                        Format: ZPL EXP IMAGING = [Im(2),Bt(1)]\n"
    "\n"
    "RAW FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Type   Optional  Description\n"
    "ESO DRS PC PROD TYPE          string No        This keyword is mandatory if the pre-processed data are used. \n"
    "                                               As the format of the zimpol data is complicated, this keyword is introduced in order \n"
    "                                               to garantee that the input frames are imaging pre-processed data, \n"
    "                                               produced by the sph_zpl_preproc_imaging recipe which added this keyword automatically. \n"
    "                                               The value of this keyword is set up to >>SPH PC PREPROC ZPL EXP IMAGING<<. \n"
    "\n"
    "DESCRIPTION:\n"
    "------------\n"
    "The recipe creates the intensity flat field calibration frames for the imaging modes.\n"
    "\n"
    "The input frames might be either intensity flat raw frames with the ZPL_INT_FLAT_IMAGING_RAW tag or pre-processed intensity flat frames,\n"
    "which should carry the ZPL_INT_FLAT_IMAGING_PREPROC_CAM1 and/or ZPL_INT_FLAT_IMAGING_PREPROC_CAM2 tags, and master bias frames (if any) with the\n"
    "ZPL_MASTER_BIAS_IMAGING_CAM1 and/or ZPL_MASTER_BIAS_IMAGING_CAM2 tags, and and master dark frames (if any) with the\n"
    "ZPL_MASTER_DARK_IMAGING_CAM1 and/or ZPL_MASTER_DARK_IMAGING_CAM2 tags.\n"
    "\n"
    "If input frames are raw frames then the intensity flat field recipe first performs the pre-processing step for all input frames (raw cubes),\n"
    "creating corresponding pre-processed frames (cubes) for both ZIMPOL cameras (see also sph_zpl_preproc_imaging for the detailed description of the\n"
    "pre-processing step).\n"
    "\n"
    "There are two main different methods to calculate the master intensity flatfield:\n"
    "\n"
    "-   combining frames (plus normalizing): in this case, the raw frames\n"
    "    must be acquired with the same DIT and filter;\n"
    "\n"
    "-   linear fitting method of the individual pixels: in this case, the\n"
    "    raw frames must be acquired either with a different\n"
    "\n"
    "    DIT or with a different intensity of the lamp, but with the same\n"
    "    filter.\n"
    "\n"
    "\n"
    "\n"
    "\n"
    "The first <combining frames method> combines pre-processed raw intensity flatfield frame (= all planes) from the pre-processed cube(s)\n"
    "using the specified collapse algorithm (usually the clean_mean algorithm, defined as a default one).\n"
    "\n"
    "After all pre-processed frames (all 4 zimpol exposure sub-frames) are combined in this way,\n"
    "the badpixel maps are determined on the results, using a simple sigma clipping algorithm.\n"
    "It sets the bad/hot pixels to be all those that are further than the\n"
    ">>specified sigma x the total RMS<< of the whole image away from the image median.\n"
    "\n"
    "Note that the badpixels that are stored in the master flat field itself as produced by this recipe contain all the badpixels (for each sub-frames\n"
    "individually) at this point in the cascade (i.e. badpixels from the master dark and master bias, if exists).\n"
    "The resulting master intensity flat field products for both cameras are then  written out in the fits files in the DOUBLE IMAGE  format:\n"
    "\n"
    "1.  zpl exp imaging odd sub-frame (informative component):\n"
    "\n"
    "    -   intensity flat field image,\n"
    "\n"
    "    -   badpixel-map,\n"
    "\n"
    "    -   ncomb-map,\n"
    "\n"
    "    -   rms-map;\n"
    "\n"
    "2.  zpl exp imaging even sub-frame (dark current component):\n"
    "\n"
    "    -   intensity flat field image\n"
    "\n"
    "    -   badpixel-map\n"
    "\n"
    "    -   ncomb-map\n"
    "\n"
    "    -   rms-map;\n"
    "\n"
    "\n"
    "\n"
    "\n"
    "\n"
    "The second <linear fitting flat fielding procedure> descibed below (identical to that for the IFS and IRDIS) is then\n"
    "applied to the each >>zpl exp imaging sub-frames<< ( odd - informative component, even-dark current component) seperately.\n"
    "\n"
    "1.  The mean value is determined for the respective sub-frame for\n"
    "\n"
    "    all exposures.\n"
    "\n"
    "2.  For every pixel $p=(x,y)$, a set of $m_i$,$v_i(x,y)$ data pairs\n"
    "\n"
    "    are stored with $m_i$ being the exposure mean value and $v_i(x,y)$\n"
    "\n"
    "    being the pixel value for exposure $i$.\n"
    "\n"
    "3.  The flat field value is defined as the slope $c_i$ of a linear fit F\n"
    "    to the data\n"
    "\n"
    "    $m_i$,$v_i$.\n"
    "\n"
    "4.  The fit itself is performed either using a maximum likelyhood method\n"
    "\n"
    "    or a robust fitting method which minimizes the sum of the absolute\n"
    "    value of\n"
    "\n"
    "    the deviations rather than the sum of the squares of the deviations\n"
    "\n"
    "    (see e.g. Numerical Recipes for the algorithm). The robust fitting\n"
    "    method will\n"
    "\n"
    "    yield better results when significant outliers (e.g. due to cosmic\n"
    "    rays) can\n"
    "\n"
    "    be expected.\n"
    "\n"
    "5.  The flat field values (linear coefficients) are saved as an image as\n"
    "    the main product of the recipe in the same\n"
    "\n"
    "    DOUBLE IMAGE format (see above).\n"
    "\n"
    "\n"
    "\n"
    "\n"
    "Aditionally, the recipe may also produce a separate output of all pixels that\n"
    "are identified as non-linear. The criteria for non-linearity are set by the user\n"
    "parameters and can be either pixels that have a flat field value outside specified\n"
    "bounds and/or pixels for which the linear fit produces a reduced chi-squared above\n"
    "a given threshold value.\n"
    "For reliable non-linearity flagging using the reduced chi-squared\n"
    "it is necessary to use many high quality input exposures.\n"
    "Since the badpixel treatment is somewhat complicated, some important points: the badpixels\n"
    "that are stored in the master flat field itself as produced by this recipe contain all the badpixels (for each sub-frames\n"
    "individually) at this point in the cascade. Pixels that\n"
    "were marked as bad from the input static badpixel map are also marked\n"
    "as bad here.\n"
    "The optional static badpixel output that is produced contains strictly only those pixel that the\n"
    "flat field recipe itself deemed to be bad. This does not necessarily include all the badpixels\n"
    "from the static badpixel input file.\n"
    "\n"
    "The intensity flat field calibration products for both cameras my be used in all subsequent imaging mode recipes if one needs to remove the\n"
    "pixel to pixel variation of the signal response on the detector.\n"
    "\n"
    "RECIPE PRODUCTS: \n"
    "-------------------- \n"
    "Tag                      Format         Description\n"
    "ZPL_INT_FLAT_FIELD_IMAGING         FITS[Im(8)]    The resulting intensity imaging flat field frame is of the DOUBLE IMAGE format.\n"
    "                                        This DOUBLE IMAGE frame contains 8 image extensions (2 master frames), grouped by the following:\n"
    "                                        odd sub-frame intensity flat field image (informative), badpixel-map, rms-map and  weight-map;\n"
    "                                        even sub-frame intensity flat field image (dark current), badpixel-map, rms-map and  weight-map.\n"
    "ZPL_INT_FLAT_FIELD_IMAGING_CAM1    FITS[Im(8)]    The resulting intensity imaging flat field frame is of the DOUBLE IMAGE format.\n"
    "                                        This DOUBLE IMAGE frame contains 8 image extensions (2 master frames), grouped by the following:\n"
    "                                        odd sub-frame intensity flat field image (informative), badpixel-map, rms-map and  weight-map;\n"
    "                                        even sub-frame intensity flat field image (dark current), badpixel-map, rms-map and  weight-map.\n"
    "ZPL_INT_FLAT_FIELD_IMAGING_CAM2    FITS[Im(8)]    The resulting intensity imaging flat field frame is of the DOUBLE IMAGE format.\n"
    "                                        This DOUBLE IMAGE frame contains 8 image extensions (2 master frames), grouped by the following:\n"
    "                                        odd sub-frame intensity flat field image (informative), badpixel-map, rms-map and  weight-map;\n"
    "                                        even sub-frame intensity flat field image (dark current), badpixel-map, rms-map and  weight-map.\n"
    "ZPL_NON_LINEAR_BADPIXELMAP_IMAGING FITS[Im(4)]    Optional output of all the non-linear pixels determined. All pixels as determined\n"
    "                                        in this recipe using the zpl.intensity_flat.badpix_low(up)tolerance parameters.\n"
    "                                        This badpixel map frame is of ZPL EXP IMAGING format:\n"
    "                                        odd sub-frame image;\n"
    "                                        even  sub-frame  image (dark current).\n"
    "ZPL_NON_LINEAR_BADPIXELMAP_IMAGING_CAM1FITS[Im(4)]    Optional output of all the non-linear pixels determined. All pixels as determined\n"
    "                                        in this recipe using the zpl.intensity_flat.badpix_low(up)tolerance parameters.\n"
    "                                        This badpixel map frame is of ZPL EXP IMAGING format:\n"
    "                                        odd sub-frame image;\n"
    "                                        even  sub-frame  image (dark current).\n"
    "ZPL_NON_LINEAR_BADPIXELMAP_IMAGING_CAM2FITS[Im(4)]    Optional output of all the non-linear pixels determined. All pixels as determined\n"
    "                                        in this recipe using the zpl.intensity_flat.badpix_low(up)tolerance parameters.\n"
    "                                        This badpixel map frame is of ZPL EXP IMAGING format:\n"
    "                                        odd sub-frame image;\n"
    "                                        even  sub-frame  image (dark current).\n"
    "\n"
    "PRODUCT FRAME KEYWORDS: \n"
    "-------------------- \n"
    "Keyword                       Frame          Type    Description\n"
    "\n"
;

/*-----------------------------------------------------------------------------
 Function code
 -----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/**
 * @defgroup sph_zpl_intensity_flat_imaging Create Master Dark Recipe
 *
 * This module provides the recipe plugin and relating functions for the
 * creation of the master dark .
 *
 * @par Synopsis:
 * @code
 *   #include "sph_zpl_intensity_flat_imaging.h"
 * @endcode
 */
/*----------------------------------------------------------------------------*/
/**@{*/

/*****************************************************************************
 *
 *
 *      First come the static and global functions necessary for the
 *      normal working of the recipe as plugin....
 *
 *
 * ***************************************************************************/
/*----------------------------------------------------------------------------*/
/**
 @brief    Build the list of available plugins, for this module.
 @param    list    the plugin list
 @return   0 if everything is ok, 1 otherwise
 @note     Only this function is exported

 Create the recipe instance and make it available to the application
 using the interface.
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe * recipe = cpl_calloc( 1, sizeof *recipe);
    cpl_plugin * plugin = &recipe->interface;

    if (cpl_plugin_init(plugin,
                        CPL_PLUGIN_API,
                        SPH_ZPL_INTENSITY_FLAT_IMAGING_VERSION,
                        CPL_PLUGIN_TYPE_RECIPE,
                        "sph_zpl_intensity_flat_imaging",
                        "Create intensity flat field, imaging mode.",
                        sph_zpl_intensity_flat_imaging_help, "Alexey Pavlov <pavlov@mpia-hd.mpg.de>",
                        "https://support.eso.org", cpl_get_license("SPHERE DRH","2012"),
                        sph_zpl_intensity_flat_imaging_create_plugin,
                        sph_zpl_intensity_flat_imaging_exec_plugin,
                        sph_zpl_intensity_flat_imaging_destroy_plugin) )
    {
        cpl_msg_error( cpl_func, "Plugin initialization failed");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }

    if (cpl_pluginlist_append(list, plugin) )
    {
        cpl_msg_error( cpl_func, "Error adding plugin to list");
        (void)cpl_error_set_where( cpl_func );
        return 1;
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Setup the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int sph_zpl_intensity_flat_imaging_create_plugin(cpl_plugin * plugin)
{
    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: An error is already set: %s",
                       cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error(cpl_func, "Null plugin");
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    if ( plugin != NULL ) {
        cpl_recipe * recipe = (cpl_recipe *)plugin;
        recipe->parameters = sph_zpl_intensity_flat_imaging_create_paramlist();
        if ( recipe->parameters == NULL ) {
            return SPH_ZPL_INTENSITY_FLAT_IMAGING_GENERAL;
        }
    }
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Testing for setup of the recipe options
 @param    plugin  the plugin
 @return   0 if everything is ok

 Wrapper to make the static plugin creation function accessible from outside
 for testing purposes.
 */
/*----------------------------------------------------------------------------*/
int sph_zpl_intensity_flat_imaging_create_test(cpl_plugin * plugin)
{
    return (int)sph_zpl_intensity_flat_imaging_create_plugin(plugin);
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Execute the plugin instance given by the interface
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_zpl_intensity_flat_imaging_exec_plugin(cpl_plugin * plugin)
{

    cpl_recipe*             recipe                  = NULL;
    cpl_error_code          recipe_code;
    sph_zpl_intensity_flat_imaging*    sph_recipe      = NULL;
    cpl_errorstate          initial_errorstate      = cpl_errorstate_get();

    if (cpl_error_get_code() != CPL_ERROR_NONE)
    {
        cpl_msg_error( cpl_func, "%s():%d: A cpl error is already set: %s",
        cpl_func, __LINE__, cpl_error_get_where());
        return (int)cpl_error_get_code();
    }

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
        return (int)cpl_error_get_code();
    }

    recipe = (cpl_recipe *)plugin;

    if (recipe->parameters == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL parameter list");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    if (recipe->frames == NULL)
    {
        cpl_msg_error( cpl_func, "Recipe invoked with NULL frame set");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
        return (int)cpl_error_get_code();
    }

    sph_recipe = sph_zpl_intensity_flat_imaging_new( recipe->frames, recipe->parameters );
    if ( sph_recipe == NULL ) {
        recipe_code = cpl_error_set_where(cpl_func);
    } else {

        recipe_code = sph_zpl_intensity_flat_imaging_run( sph_recipe );

        sph_zpl_intensity_flat_imaging_delete( sph_recipe );
    }

    if ( !cpl_errorstate_is_equal(initial_errorstate) )
    {
        cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);
    }
    sph_end_erex(); //close the sphere system
    return (int)(recipe_code >= CPL_ERROR_EOL /* Workaround esorex PIPE-6887 */
                 ? CPL_ERROR_EOL : recipe_code);
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Destroy what has been created by the 'create' function
 @param    plugin  the plugin
 @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int sph_zpl_intensity_flat_imaging_destroy_plugin(cpl_plugin * plugin)
{
    cpl_recipe * recipe;

    if (plugin == NULL)
    {
        cpl_msg_error( cpl_func, "Null plugin");
        cpl_ensure_code( 0, (int)CPL_ERROR_NULL_INPUT );
    }

    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE)
    {
        cpl_msg_error( cpl_func, "Plugin is not a recipe");
        cpl_ensure_code( 0, (int)CPL_ERROR_TYPE_MISMATCH );
    }

    recipe = (cpl_recipe *)plugin;

    cpl_parameterlist_delete(recipe->parameters);

    return 0;
}

/*****************************************************************************
 *
 *
 *      Here come the "Member" functions of the sph_zpl_intensity_flat_imaging struct
 *
 *
 * ***************************************************************************/

/*----------------------------------------------------------------------------*/
/**
 @brief    Fill a pre-allocated parameterlist for the recipe
 @param    result A pre-allocated parameterlist
 @return   CPL_ERROR_NONE iff successful

 This function fills the standard parameterlist for the recipe

 */
/*----------------------------------------------------------------------------*/
static cpl_error_code sph_zpl_intensity_flat_imaging_fill_parameterlist(cpl_parameterlist * result)
{
    cpl_parameter* p;

        /* Code to set up parameters GENERATED DO NOT EDIT */

        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.outfilename",
                                   CPL_TYPE_STRING,
                                   "The output filename for the product, camera-1/2. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"zpl.intensity_flat_imaging",
                                   "zpl_intensity_flat_imaging.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.outfilename_cam1",
                                   CPL_TYPE_STRING,
                                   "The output filename for the product, camera-1. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"zpl.intensity_flat_imaging",
                                   "zpl_intensity_flat_imaging_cam1.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.outfilename_cam2",
                                   CPL_TYPE_STRING,
                                   "The output filename for the product, camera-2. Please also see the esorex documentation "
                                   "for naming of output products. "
                                   ,"zpl.intensity_flat_imaging",
                                   "zpl_intensity_flat_imaging_cam2.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.subtract_overscan",
                                   CPL_TYPE_BOOL,
                                   "Flag to set if the overscan mean values must be subtracted from pre-processed data (TRUE) "
                                   "Note that this parameter is applied if pre-processed data containt overscan table "
                                   ,"zpl.intensity_flat_imaging",
                                   1 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.badpixfilename",
                                   CPL_TYPE_STRING,
                                   "Controls the filename of the badpixel map, if requested for output. Ignored if make_badpix is FALSE. "
                                   ,"zpl.intensity_flat_imaging",
                                   "zpl_intensity_flat_imaging_nonlin_badpixels.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.badpixfilename_cam1",
                                   CPL_TYPE_STRING,
                                   "Controls the filename of the badpixel map, if requested for output. Ignored if make_badpix is FALSE. "
                                   ,"zpl.intensity_flat_imaging",
                                   "zpl_intensity_flat_imaging_nonlin_badpixels_cam1.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.badpixfilename_cam2",
                                   CPL_TYPE_STRING,
                                   "Controls the filename of the badpixel map, if requested for output. Ignored if make_badpix is FALSE. "
                                   ,"zpl.intensity_flat_imaging",
                                   "zpl_intensity_flat_imaging_nonlin_badpixels_cam2.fits" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.robust_fit",
                                   CPL_TYPE_BOOL,
                                   "Controls if fitting method is to be a robust linear fit. This will reduce the effect of cosmic "
                                   "rays and other temporary bad pixels. See e.g. Numerical Recipes for a description of the algorithm "
                                   ,"zpl.intensity_flat_imaging",
                                   0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.collapse",
                                   CPL_TYPE_BOOL,
                                   "Controls if the collapse is used to calculate intensity flat instead of the fitting "
                                   ,"zpl.intensity_flat_imaging",
                                   1 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_enum("zpl.intensity_flat_imaging.coll_alg",
                                   CPL_TYPE_INT,
                                   "Set the collapse algorithm. The available algorithms: 0 = Mean, 1 = Median, 2 = Clean Mean. Default is 2 = Clean Mean "
                                   ,"zpl.intensity_flat_imaging",
                                   2 ,
                                   3,0,1,2);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("zpl.intensity_flat_imaging.coll_alg.clean_mean.reject_high",
                                   CPL_TYPE_INT,
                                   "The number of pixels to reject when combining frames at the high end. Number of input frames must be > reject_high +reject_low "
                                   ,"zpl.intensity_flat_imaging",
                                   0 ,
                                   0,20);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("zpl.intensity_flat_imaging.coll_alg.clean_mean.reject_low",
                                   CPL_TYPE_INT,
                                   "The number of pixels to reject when combining frames at the low end. Number of input frames must be > reject_high +reject_low "
                                   ,"zpl.intensity_flat_imaging",
                                   0 ,
                                   0,20);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_range("zpl.intensity_flat_imaging.sigma_clip",
                                   CPL_TYPE_DOUBLE,
                                   "The sigma clipping value for static badpixel detection. Default is 5. "
                                   ,"zpl.intensity_flat_imaging",
                                   5.0 ,
                                   0.0,200.0);

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.badpix_lowtolerance",
                                   CPL_TYPE_DOUBLE,
                                   "Threshold value for linearity badpixels. "
                                   "All pixels that have a flat field (slope) below this value will be flagged as bad. "
                                   ,"zpl.intensity_flat_imaging",
                                   0.1 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.badpix_uptolerance",
                                   CPL_TYPE_DOUBLE,
                                   "Threshold value for linearity badpixels. "
                                   "All pixels that have a flat field (slope) above this value will be flagged as bad. "
                                   ,"zpl.intensity_flat_imaging",
                                   10.0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.badpix_chisqtolerance",
                                   CPL_TYPE_DOUBLE,
                                   "Threshold value for linearity badpixels. All pixels that have chi-squared value for "
                                   "the linear fit that is above this value will be flagged as bad "
                                   ,"zpl.intensity_flat_imaging",
                                   50.0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.intensity_flat_imaging.keep_intermediate",
                                   CPL_TYPE_BOOL,
                                   "Flag to set if intermediate date must be saved, namely pre-processed and overscan pre-processed subtracted data, linbadpix map and non-normalized products (FALSE) "
                                   ,"zpl.intensity_flat_imaging",
                                   0 );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.preproc.outfilename_cam1",
                                   CPL_TYPE_STRING,
                                   "The postfix-  of the intermediate filename of the pre-processed raw data for the CAMERA-1. "
                                   ,"zpl.preproc",
                                   "preproc_cam1" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        p = cpl_parameter_new_value("zpl.preproc.outfilename_cam2",
                                   CPL_TYPE_STRING,
                                   "The postfix- of the  intermediate filename of the pre-processed raw data for the CAMERA-2. "
                                   ,"zpl.preproc",
                                   "preproc_cam2" );

        cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
        cpl_parameterlist_append(result, p);
        /* End of code to set up parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/

    return cpl_error_set_where(cpl_func);
}
/*----------------------------------------------------------------------------*/
/**
 @brief  Create and return the standard parameterlist for the recipe
 @return The standard parameterlist or NULL on error
 @note   The returned object must be deallocated using cpl_parameterlist_delete

 */
/*----------------------------------------------------------------------------*/
cpl_parameterlist* sph_zpl_intensity_flat_imaging_create_paramlist(void)
{
    cpl_parameterlist* self = cpl_parameterlist_new();

    if (sph_zpl_intensity_flat_imaging_fill_parameterlist(self)) {
        cpl_parameterlist_delete(self);
        self = NULL;
        (void)cpl_error_set_where(cpl_func);
    }

    return self;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe frames, and assign them to the sph_zpl_intensity_flat_imaging
           structure.
 @param    self     the sph_zpl_intensity_flat_imaging recipe structure
 @return   error code of the check.

 This checks the frames that were given in the sph_zpl_intensity_flat_imaging_new function
 as parameters for the correctness and completeness. If the frames needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code  sph_zpl_intensity_flat_imaging_check_frames( sph_zpl_intensity_flat_imaging* self )
{
    sph_error_code           rerr            = CPL_ERROR_NONE;
    /* Code to check frames GENERATED DO NOT EDIT */

    cpl_frame* aframe;
    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_IFF_IMAGING_RAW );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_RAW );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->rawframes = sph_utils_extract_frames( self->inframes, SPH_ZPL_TAG_IFF_IMAGING_RAW );
    if ( ! self->rawframes )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract rawframes frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_IFF_IMAGING_RAW);
        cpl_error_reset();
    }

    else {

         int numframes = (int)cpl_frameset_get_size( self->rawframes );
         for ( int ii = 0; ii < numframes; ++ii ) {
             cpl_frame* iframe = cpl_frameset_get_position( self->rawframes, ii );
             const char* ifname = cpl_frame_get_filename(iframe);
             cpl_propertylist* pl = cpl_propertylist_load(ifname, 0);
             if (pl == NULL ) {
                 sph_error_raise(cpl_error_get_code(),
                    __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                 "Could not read keywords from file %s. "
                 "Either the file is not readable or corrupted. ",
                  ifname);
             }
             else {
                 rerr = cpl_frame_set_group( iframe, CPL_FRAME_GROUP_RAW );
                 int haskey, okval;
                 haskey = 0;okval = 0;
                 if ( cpl_propertylist_has(pl,"ESO DRS PC PROD TYPE") )  {
                     haskey = 1;
                     if ( cpl_propertylist_get_type(pl,"ESO DRS PC PROD TYPE") == CPL_TYPE_STRING )  {
                         okval = 1; //Type is string so always ok
                     } 
                     if ( haskey == 0 || okval == 0 ) {
                         sph_error_raise(CPL_ERROR_ILLEGAL_INPUT,
                            __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                         "Could not read a keyword ESO DRS PC PROD TYPE from file %s. "
                         "that has __type string "
                         "and values between 0.0 and 0.0 "
                         "The recipe requires this. ",
                         ifname);
                     }
                 }
                 cpl_propertylist_delete(pl); pl=NULL;
             }
         }
         numframes -= (int)cpl_frameset_get_size( self->rawframes );
         assert( numframes == 0);
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_IFF_IMAGING_PREPROC_RAW );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_RAW );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->preproc_frames = sph_utils_extract_frames( self->inframes, SPH_ZPL_TAG_IFF_IMAGING_PREPROC_RAW );
    if ( ! self->preproc_frames )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract preproc_frames frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_IFF_IMAGING_PREPROC_RAW);
        cpl_error_reset();
    }

    else {

         int numframes = (int)cpl_frameset_get_size( self->preproc_frames );
         for ( int ii = 0; ii < numframes; ++ii ) {
             cpl_frame* iframe = cpl_frameset_get_position( self->preproc_frames, ii );
             const char* ifname = cpl_frame_get_filename(iframe);
             cpl_propertylist* pl = cpl_propertylist_load(ifname, 0);
             if (pl == NULL ) {
                 sph_error_raise(cpl_error_get_code(),
                    __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                 "Could not read keywords from file %s. "
                 "Either the file is not readable or corrupted. ",
                  ifname);
             }
             else {
                 rerr = cpl_frame_set_group( iframe, CPL_FRAME_GROUP_RAW );
                 int haskey, okval;
                 haskey = 0;okval = 0;
                 if ( cpl_propertylist_has(pl,"ESO DRS PC PROD TYPE") )  {
                     haskey = 1;
                     if ( cpl_propertylist_get_type(pl,"ESO DRS PC PROD TYPE") == CPL_TYPE_STRING )  {
                         okval = 1; //Type is string so always ok
                     } 
                     if ( haskey == 0 || okval == 0 ) {
                         sph_error_raise(CPL_ERROR_ILLEGAL_INPUT,
                            __FILE__, __func__, __LINE__ ,SPH_ERROR_ERROR,
                         "Could not read a keyword ESO DRS PC PROD TYPE from file %s. "
                         "that has __type string "
                         "and values between 0.0 and 0.0 "
                         "The recipe requires this. ",
                         ifname);
                     }
                 }
                 cpl_propertylist_delete(pl); pl=NULL;
             }
         }
         numframes -= (int)cpl_frameset_get_size( self->preproc_frames );
         assert( numframes == 0);
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_IFF_IMAGING_PREPROC_CAM1 );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->preproc_frames_cam1 = sph_utils_extract_frames( self->inframes, SPH_ZPL_TAG_IFF_IMAGING_PREPROC_CAM1 );
    if ( ! self->preproc_frames_cam1 )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract preproc_frames_cam1 frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_IFF_IMAGING_PREPROC_CAM1);
        cpl_error_reset();
    }

    else {

         int numframes = (int)cpl_frameset_get_size( self->preproc_frames_cam1 );
         for ( int ii = 0; ii < numframes; ++ii ) {
         }
         numframes -= (int)cpl_frameset_get_size( self->preproc_frames_cam1 );
         assert( numframes == 0);
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_IFF_IMAGING_PREPROC_CAM2 );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->preproc_frames_cam2 = sph_utils_extract_frames( self->inframes, SPH_ZPL_TAG_IFF_IMAGING_PREPROC_CAM2 );
    if ( ! self->preproc_frames_cam2 )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract preproc_frames_cam2 frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_IFF_IMAGING_PREPROC_CAM2);
        cpl_error_reset();
    }

    else {

         int numframes = (int)cpl_frameset_get_size( self->preproc_frames_cam2 );
         for ( int ii = 0; ii < numframes; ++ii ) {
         }
         numframes -= (int)cpl_frameset_get_size( self->preproc_frames_cam2 );
         assert( numframes == 0);
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_BIAS_IMAGING_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_bias_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_BIAS_IMAGING_CALIB );
    if ( ! self->master_bias_frame )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract master_bias_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_BIAS_IMAGING_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_bias_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_BIAS_IMAGING_CALIB_CAM1 );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_bias_frame_cam1 = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_BIAS_IMAGING_CALIB_CAM1 );
    if ( ! self->master_bias_frame_cam1 )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract master_bias_frame_cam1 frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_BIAS_IMAGING_CALIB_CAM1);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_bias_frame_cam1, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_BIAS_IMAGING_CALIB_CAM2 );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_bias_frame_cam2 = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_BIAS_IMAGING_CALIB_CAM2 );
    if ( ! self->master_bias_frame_cam2 )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract master_bias_frame_cam2 frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_BIAS_IMAGING_CALIB_CAM2);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_bias_frame_cam2, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_DARK_IMAGING_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_dark_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_DARK_IMAGING_CALIB );
    if ( ! self->master_dark_frame )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract master_dark_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_DARK_IMAGING_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_dark_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_DARK_IMAGING_CALIB_CAM1 );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_dark_frame_cam1 = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_DARK_IMAGING_CALIB_CAM1 );
    if ( ! self->master_dark_frame_cam1 )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract master_dark_frame_cam1 frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_DARK_IMAGING_CALIB_CAM1);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_dark_frame_cam1, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_DARK_IMAGING_CALIB_CAM2 );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->master_dark_frame_cam2 = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_DARK_IMAGING_CALIB_CAM2 );
    if ( ! self->master_dark_frame_cam2 )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract master_dark_frame_cam2 frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_DARK_IMAGING_CALIB_CAM2);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->master_dark_frame_cam2, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_STATIC_BADPIXELMAP_IMAGING_CALIB );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->static_badpixel_frame = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_STATIC_BADPIXELMAP_IMAGING_CALIB );
    if ( ! self->static_badpixel_frame )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract static_badpixel_frame frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_STATIC_BADPIXELMAP_IMAGING_CALIB);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->static_badpixel_frame, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_STATIC_BADPIXELMAP_IMAGING_CALIB_CAM1 );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->static_badpixel_frame_cam1 = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_STATIC_BADPIXELMAP_IMAGING_CALIB_CAM1 );
    if ( ! self->static_badpixel_frame_cam1 )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract static_badpixel_frame_cam1 frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_STATIC_BADPIXELMAP_IMAGING_CALIB_CAM1);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->static_badpixel_frame_cam1, CPL_FRAME_GROUP_CALIB );
    }

    aframe = cpl_frameset_find( self->inframes, SPH_ZPL_TAG_STATIC_BADPIXELMAP_IMAGING_CALIB_CAM2 );
    while ( aframe ) {
        cpl_frame_set_group( aframe, CPL_FRAME_GROUP_CALIB );
        aframe = cpl_frameset_find( self->inframes, NULL );
    }
    self->static_badpixel_frame_cam2 = sph_utils_extract_frame( self->inframes, SPH_ZPL_TAG_STATIC_BADPIXELMAP_IMAGING_CALIB_CAM2 );
    if ( ! self->static_badpixel_frame_cam2 )
    {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_FRAMES_MISSING,
                         __FILE__, __func__, __LINE__ ,
                         SPH_ERROR_INFO,
                         "Could not extract static_badpixel_frame_cam2 frames."
                         "Since this is an optional frame, this is ok "
                         "and this message is just informational. If you intended, "
                         "to use them check that they have the %s tag.",
                         SPH_ZPL_TAG_STATIC_BADPIXELMAP_IMAGING_CALIB_CAM2);
        cpl_error_reset();
    }

    else {

         rerr = cpl_frame_set_group( self->static_badpixel_frame_cam2, CPL_FRAME_GROUP_CALIB );
    }

    /* End of code to check frames BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    self->current_raw_frameset = sph_utils_extract_frames_group( self->inframes, CPL_FRAME_GROUP_RAW );

    return rerr;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Check the recipe parameters, and assign them to the sph_zpl_intensity_flat_imaging
           structure.
 @param    self     the sph_zpl_intensity_flat_imaging recipe structure
 @return   error code of the check.

 This checks the parameters that were given in the sph_zpl_intensity_flat_imaging_new function
 as parameters for the correctness and completeness. If the parameters needed are
 found they are also assigned to the corresponding structure members.

 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_zpl_intensity_flat_imaging_check_params( sph_zpl_intensity_flat_imaging* self ) {
    sph_error_code                 rerr               = CPL_ERROR_NONE;
    const cpl_parameter*           param              = NULL;

    if ( self->framecomb_parameterlist != NULL ) {
        cpl_parameterlist_delete( self->framecomb_parameterlist );
        self->framecomb_parameterlist = NULL;
    }
    self->framecomb_parameterlist = cpl_parameterlist_new();
    /* Code to check parameters GENERATED DO NOT EDIT */

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.outfilename");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.outfilename");
    }

    self->intensity_flat_imaging_outfilename = cpl_parameter_get_string(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.outfilename_cam1");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.outfilename_cam1");
    }

    self->intensity_flat_imaging_outfilename_cam1 = cpl_parameter_get_string(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.outfilename_cam2");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.outfilename_cam2");
    }

    self->intensity_flat_imaging_outfilename_cam2 = cpl_parameter_get_string(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.subtract_overscan");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.subtract_overscan");
    }

    self->subtract_overscan = cpl_parameter_get_bool(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.badpixfilename");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.badpixfilename");
    }

    self->badpix_filename = cpl_parameter_get_string(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.badpixfilename_cam1");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.badpixfilename_cam1");
    }

    self->badpix_filename_cam1 = cpl_parameter_get_string(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.badpixfilename_cam2");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.badpixfilename_cam2");
    }

    self->badpix_filename_cam2 = cpl_parameter_get_string(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.robust_fit");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.robust_fit");
    }

    self->robust_fit = cpl_parameter_get_bool(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.collapse");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.collapse");
    }

    self->collapse = cpl_parameter_get_bool(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.coll_alg");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.coll_alg");
    }

    self->coll_alg = cpl_parameter_get_int(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.coll_alg.clean_mean.reject_high");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.coll_alg.clean_mean.reject_high");
    }

    self->clean_mean_reject_high = cpl_parameter_get_int(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    cpl_parameterlist_append( self->framecomb_parameterlist,
                              cpl_parameter_new_range( "clean_mean.reject_high",
                                                       CPL_TYPE_INT,
                                                       NULL,
                                                       NULL,
                                                       cpl_parameter_get_int( param ) ,
                                                       0,20
                                                      )
                            );

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.coll_alg.clean_mean.reject_low");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.coll_alg.clean_mean.reject_low");
    }

    self->clean_mean_reject_low = cpl_parameter_get_int(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    cpl_parameterlist_append( self->framecomb_parameterlist,
                              cpl_parameter_new_range( "clean_mean.reject_low",
                                                       CPL_TYPE_INT,
                                                       NULL,
                                                       NULL,
                                                       cpl_parameter_get_int( param ) ,
                                                       0,20
                                                      )
                            );

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.sigma_clip");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.sigma_clip");
    }

    self->sigma_clip = cpl_parameter_get_double(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.badpix_lowtolerance");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.badpix_lowtolerance");
    }

    self->badpix_lowtolerance = cpl_parameter_get_double(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.badpix_uptolerance");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.badpix_uptolerance");
    }

    self->badpix_uptolerance = cpl_parameter_get_double(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.badpix_chisqtolerance");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.badpix_chisqtolerance");
    }

    self->badpix_chisqtolerance = cpl_parameter_get_double(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.intensity_flat_imaging.keep_intermediate");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.intensity_flat_imaging.keep_intermediate");
    }

    self->keep_intermediate = cpl_parameter_get_bool(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.preproc.outfilename_cam1");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.preproc.outfilename_cam1");
    }

    self->outfilename_cam1 = cpl_parameter_get_string(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    param = cpl_parameterlist_find_const( self->inparams,
                                          "zpl.preproc.outfilename_cam2");
    if (param == NULL) {
        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
                                     "Recipe parameter missing: "
                                     "zpl.preproc.outfilename_cam2");
    }

    self->outfilename_cam2 = cpl_parameter_get_string(param);
    if (cpl_error_get_code()) {
        return cpl_error_set_where(cpl_func);
    }

    /* End of code to check parameters BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/

    return rerr;
}
/*----------------------------------------------------------------------------*/
/**
 @brief    Create a new sph_zpl_intensity_flat_imaging structure.

 @param    frameset     the input recipe frames
 @param    parlist      the input recipe parameters

 @return   pointer to newly created structure or NULL in case of error

 This creates a new sph_zpl_intensity_flat_imaging_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_zpl_intensity_flat_imaging* sph_zpl_intensity_flat_imaging_new( cpl_frameset * frameset,
                                              cpl_parameterlist * parlist )
{
    sph_zpl_intensity_flat_imaging*    result      = NULL;
    int                     rerr        = CPL_ERROR_NONE;

    sph_init_erex();
    if ( frameset == NULL || parlist == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return NULL;
    }

    result = cpl_calloc( 1, sizeof(sph_zpl_intensity_flat_imaging) );

    if ( result == NULL ) {
        sph_error_raise( SPH_ZPL_INTENSITY_FLAT_IMAGING_GENERAL, __FILE__, __func__,
                         __LINE__, SPH_ERROR_ERROR,
                         "Could not allocate the structure.");
        return result;
    }

    result->inframes = frameset;
    result->inparams = parlist;

    rerr = sph_zpl_intensity_flat_imaging_check_frames( result );
    if ( rerr == CPL_ERROR_NONE) rerr = sph_zpl_intensity_flat_imaging_check_params( result );

    if ( rerr != CPL_ERROR_NONE ) {
        if ( result ) {
            sph_zpl_intensity_flat_imaging_delete( result );
        }
        result = NULL;
    }

    return result;
}

/*----------------------------------------------------------------------------*/
/**
 @brief    Delete the sph_zpl_intensity_flat_imaging structure.

 @param    self         the structure tp delete

 @return   error code of the operation

 This creates a new sph_zpl_intensity_flat_imaging_structure and fills the inparams and
 inframes fields with the pointers pf frameset and parlist.
 The return value is the error code of the check.
 */
/*----------------------------------------------------------------------------*/
sph_error_code sph_zpl_intensity_flat_imaging_delete( sph_zpl_intensity_flat_imaging* self )
{
    sph_error_code          rerr            = CPL_ERROR_NONE;

    if ( self == NULL ) {
        sph_error_raise( CPL_ERROR_NULL_INPUT, __FILE__,
                         __func__, __LINE__,
                         SPH_ERROR_ERROR,
                         "Null input pointer.");
        return rerr;
    }

    if ( self->framecomb_parameterlist != NULL ) {
        cpl_parameterlist_delete( self->framecomb_parameterlist );
        self->framecomb_parameterlist = NULL;
    }

    /* Code to delete recipe pointers GENERATED DO NOT EDIT */

    if ( self->rawframes != NULL ) {
        cpl_frameset_delete( self->rawframes );
        self->rawframes = NULL;
    }
    if ( self->preproc_frames != NULL ) {
        cpl_frameset_delete( self->preproc_frames );
        self->preproc_frames = NULL;
    }
    if ( self->preproc_frames_cam1 != NULL ) {
        cpl_frameset_delete( self->preproc_frames_cam1 );
        self->preproc_frames_cam1 = NULL;
    }
    if ( self->preproc_frames_cam2 != NULL ) {
        cpl_frameset_delete( self->preproc_frames_cam2 );
        self->preproc_frames_cam2 = NULL;
    }
    if ( self->master_bias_frame != NULL ) {
        cpl_frame_delete( self->master_bias_frame );
        self->master_bias_frame = NULL;
    }
    if ( self->master_bias_frame_cam1 != NULL ) {
        cpl_frame_delete( self->master_bias_frame_cam1 );
        self->master_bias_frame_cam1 = NULL;
    }
    if ( self->master_bias_frame_cam2 != NULL ) {
        cpl_frame_delete( self->master_bias_frame_cam2 );
        self->master_bias_frame_cam2 = NULL;
    }
    if ( self->master_dark_frame != NULL ) {
        cpl_frame_delete( self->master_dark_frame );
        self->master_dark_frame = NULL;
    }
    if ( self->master_dark_frame_cam1 != NULL ) {
        cpl_frame_delete( self->master_dark_frame_cam1 );
        self->master_dark_frame_cam1 = NULL;
    }
    if ( self->master_dark_frame_cam2 != NULL ) {
        cpl_frame_delete( self->master_dark_frame_cam2 );
        self->master_dark_frame_cam2 = NULL;
    }
    if ( self->static_badpixel_frame != NULL ) {
        cpl_frame_delete( self->static_badpixel_frame );
        self->static_badpixel_frame = NULL;
    }
    if ( self->static_badpixel_frame_cam1 != NULL ) {
        cpl_frame_delete( self->static_badpixel_frame_cam1 );
        self->static_badpixel_frame_cam1 = NULL;
    }
    if ( self->static_badpixel_frame_cam2 != NULL ) {
        cpl_frame_delete( self->static_badpixel_frame_cam2 );
        self->static_badpixel_frame_cam2 = NULL;
    }
 
    sph_double_image_delete(self->intensity_flat_field_imaging);
    sph_double_image_delete(self->intensity_flat_field_imaging_cam1);
    sph_double_image_delete(self->intensity_flat_field_imaging_cam2);
    /* End of code to delete pointers BUT ITS STILL NOT SAFE TO EDIT! FILE IS GENERATED !*/
    if ( self->current_raw_frameset ) {
        cpl_frameset_delete(self->current_raw_frameset);
        self->current_raw_frameset = NULL;
    }
    sph_polygon_free_all();
    cpl_free(    self);
    return rerr;

}


/**@}*/
