/* $Id: $
 * This file is part of the SPHERE Pipeline
 * Copyright (C) 2007-2010 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: $
 * $Date: $
 * $Revision: $
 * $Name: $
 */

#ifndef SPH_FRAMESORTER_H_
#define SPH_FRAMESORTER_H_
#include "cpl.h"
#include "sph_error.h"

//const int SPH_FRAMESORT_MAX_SORT;

typedef struct _sph_framebag_ {
	cpl_frameset*		frameset;
	double*				mins;
	double*				maxs;
	cpl_type*			types;
	short				nsort;
	struct _sph_framebag_*		next;
} sph_framebag;

typedef struct _sph_framesorter_ {
	sph_framebag*		framebag_list;
	cpl_frameset*		inframeset;
	cpl_parameterlist*	sort_criteria;
	int					nsets;
} sph_framesorter;

sph_framebag* sph_framebag_new( int nsort )
           CPL_ATTR_ALLOC;

sph_framebag* sph_framebag_get_last( sph_framebag* self );

sph_framebag* sph_framebag_search( sph_framebag* self, const cpl_vector* svect );


void sph_framebag_delete_all( sph_framebag* self );

void sph_framebag_delete( sph_framebag* self );


sph_framesorter*
sph_framesorter_new( cpl_frameset* inframes );
int
sph_framesorter_check_set( sph_framesorter* self, cpl_frame* inframe, int inpos );
sph_error_code
sph_framesorter_sort( sph_framesorter* self,
		cpl_parameterlist* dependencies );
sph_framebag* sph_framesort_new_bag( sph_framesorter* self, cpl_vector* validvals ) ;
cpl_frameset*
sph_framesorter_get_frameset( sph_framesorter* self,
		int bagid );

void sph_framesorter_delete( sph_framesorter* self );

#endif /* SPH_FRAMESORTER_H_ */
